// ADOBE SYSTEMS INCORPORATED
// Copyright  1993 - 2002 Adobe Systems Incorporated
// All Rights Reserved
//
// NOTICE:  Adobe permits you to use, modify, and distribute this 
// file in accordance with the terms of the Adobe license agreement
// accompanying it.  If you have received this file from a source
// other than Adobe, then your use, modification, or distribution
// of it requires the prior written permission of Adobe.
//-------------------------------------------------------------------
//-------------------------------------------------------------------------------
//
//	File:
//		NearestBase.cpp
//
//	Description:
//		This file contains the source and functions
//		for the Color Picker example module NearestBase.
//
//	Use:
//		NearestBase provides another color picker (available
//		from File: Preferences: General) that cycles through
//		four primary colors.  Once selected, clicking any
//		location that normally pops a picker (such as the
//		foreground or background color) will automatically
//		cycle to the next primary color.
//
//
//-------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
//	Includes
//-------------------------------------------------------------------------------

#include "NearestBase.h"

//-------------------------------------------------------------------------------
//	Globals -- Define global variables for plug-in scope.
//-------------------------------------------------------------------------------
SPBasicSuite *	sSPBasic = NULL;
Ptr				proxyBuffer;
GPtr			globals = NULL; 		// actual globals

BYTE			fastcolor[4];
BYTE			curcolor[4];
double			cursikisou=0;
const int		circleCx =96;		//T[ÑZ^[
const int		circleCy =120;
int				deltacursorx=circleCx;
int				deltacursory=circleCy;
int				hensyuu=0;


extern void RGB2HSB( double max, double mid, double min, double r, double g, double b, double* h, double* s, double* v, int bitlength, int feeld);


//-------------------------------------------------------------------------------
//
//	PluginMain / main
//
//	All calls to the plug-in module come through this routine.
//	It must be placed first in the resource.  To achieve this,
//	most development systems require this be the first routine
//	in the source.
//
//	The entrypoint will be "pascal void" for Macintosh,
//	"void" for Windows.
//
//	Inputs:
//		const int16 selector				Host provides selector indicating
//											what command to do.
//
//		PIPickerParams *pickerParamBlock	Host provides pointer to parameter
//											block containing pertinent data
//											and callbacks from the host.
//											See PIPicker.h.
//
//	Outputs:
//		PIPickerParams * pickerParamBlock	Host provides pointer to parameter
//											block. Use it to pass pertinent
//											data to and from the host.
//											See PIPicker.h.
//
//		intptr_t *data						Use this to store a handle or pointer to our
//											global parameters structure, which
//											is maintained by the host between
//											calls to the plug-in.
//
//		int16 *result						Return error result or noErr.  Some
//											errors are handled by the host, some
//											are silent, and some you must handle.
//											See PIGeneral.h.
//
//-------------------------------------------------------------------------------
DLLExport MACPASCAL void PluginMain (const int16 selector,
						             PIPickerParams * pickerParamBlock,
						             intptr_t * data,
						             int16 * result)
{
	//---------------------------------------------------------------------------
	//	(1) Check for about box request.
	//
	// 	The about box is a special request; the parameter block is not filled
	// 	out, none of the callbacks or standard data is available.  Instead,
	// 	the parameter block points to an AboutRecord.
	//---------------------------------------------------------------------------

	if (selector == pickerSelectorAbout)
	{
		sSPBasic = ((AboutRecord*)pickerParamBlock)->sSPBasic;
		DoAbout((AboutRecordPtr)pickerParamBlock);
	}
	else
	{ // do the rest of the process as normal:

		sSPBasic = ((PIPickerParams*)pickerParamBlock)->sSPBasic;

		Ptr globalPtr = NULL;		// Pointer for global structure

		//-----------------------------------------------------------------------
		//	(2) Allocate and initalize globals.
		//
		// 	AllocateGlobals requires the pointer to result, the pointer to the
		// 	parameter block, a pointer to the handle procs, the size of our local
		// 	"Globals" structure, a pointer to the long *data, a Function
		// 	Proc (FProcP) to the InitGlobals routine.  It automatically sets-up,
		// 	initializes the globals (if necessary), results result to 0, and
		// 	returns with a valid pointer to the locked globals handle or NULL.
		//-----------------------------------------------------------------------
		
		globalPtr = AllocateGlobals (result,
									 pickerParamBlock,
									 pickerParamBlock->handleProcs,
									 sizeof(Globals),
						 			 data,
						 			 InitGlobals);
		
		if (globalPtr == NULL)
		{ // Something bad happened if we couldn't allocate our pointer.
		  // Fortunately, everything's already been cleaned up,
		  // so all we have to do is report an error.
		  
		  *result = memFullErr;
		  return;
		}
		
		// Get our "globals" variable assigned as a Global Pointer struct with the
		// data we've returned:
		globals = (GPtr)globalPtr;
		BufferID proxyBufferID = NULL;
		globals->pickerParamBlock->bufferProcs->allocateProc(300*300*4, &proxyBufferID);
		proxyBuffer = globals->pickerParamBlock->bufferProcs->lockProc(proxyBufferID, true);



		fastcolor[0]=globals->pickerParamBlock->pickParms.colorComponents[0];
		fastcolor[1]=globals->pickerParamBlock->pickParms.colorComponents[1];
		fastcolor[2]=globals->pickerParamBlock->pickParms.colorComponents[2];
		curcolor[0]=globals->pickerParamBlock->pickParms.colorComponents[0];
		curcolor[1]=globals->pickerParamBlock->pickParms.colorComponents[1];
		curcolor[2]=globals->pickerParamBlock->pickParms.colorComponents[2];



		{
			double		r=(double)fastcolor[0];
			double		g=(double)fastcolor[1];
			double		b=(double)fastcolor[2];
			double		v;
			double		s;
			double		h;
			double		max,mid,min;
			int			feeld;
			const int	bitlength=0xFF;

			if(r>=g&&r>=b){
				if(g>=b){
					max=r; mid=g; min=b;
					feeld=0;
				}
				else {
					max=r; mid=b; min=g;
					feeld=0;
				}
			}
			else if(g>=b&&g>=r){
				if(b>=r){
					max=g; mid=b; min=r;
					feeld=1;
				}
				else {
					max=g; mid=r; min=b;
					feeld=1;
				}
			}
			else if(b>=r&&b>=g){
				if(r>=g){
					max=b; mid=r; min=g;
					feeld=2;
				}
				else {
					max=b; mid=g; min=r;
					feeld=2;
				}
			}

			RGB2HSB( max, mid, min, r,g,b, &h, &s, &v, bitlength, feeld);
			cursikisou=((int)h+360+180+45)%360;


			int ofsx=59;
			int ofsy=69;
			if(v<=0.0000001){v=0.0000001;}
			if(s<=0.0000001){s=0.0000001;}
			v=v; s=1-s;

			deltacursorx=(int)((v/100)*(s*100)*100);
			deltacursory=(int)( 100-(100)*(v) ) + (deltacursorx/2);		// Op̌ƂlȂĂ񂾂ȥ100*xł݂PɁB
			
			deltacursorx+=ofsx;
			deltacursory+=ofsy;

		}



		//-----------------------------------------------------------------------
		//	(3) Run the plug in's main routine.
		//
		//-----------------------------------------------------------------------	
		DoParameters(globals);

		// ҏWĂXViҏWĖĂނƔɐF̂łh~邽߁j
		if(hensyuu=1){
			gStuff->pickParms.resultSpace = plugIncolorServicesRGBSpace;
			gStuff->pickParms.colorComponents[0] = curcolor[0] *0x101;
			gStuff->pickParms.colorComponents[1] = curcolor[1] *0x101;
			gStuff->pickParms.colorComponents[2] = curcolor[2] *0x101;
			gStuff->pickParms.colorComponents[3] = 255 * 0x101;
		}

		//-----------------------------------------------------------------------
		//	(4) Unlock data, and exit resource.
		//
		//	Result is automatically returned in *result, which is
		//	pointed to by gResult.
		//-----------------------------------------------------------------------	
		
		// unlock handle pointing to parameter block and data so it can move
		// if memory gets shuffled:
		if ((Handle)*data != NULL)
			PIUnlockHandle((Handle)*data);


//		globals->pickerParamBlock->bufferProcs->unlockProc(proxyBufferID);
//		globals->pickerParamBlock->bufferProcs->freeProc(  proxyBufferID);
//		proxyBufferID = NULL;
//		proxyBuffer   = NULL;
//		*result=NULL;

	
	} // about selector special		
	
} // end PluginMain





void RGB2HSB( double max, double mid, double min, double r, double g, double b, double* h, double* s, double* v, int bitlength, int feeld)
{
	if(max<=0.0000001){max=0.0000001;}
	*v=max/bitlength;
	
	*s=(max-min)/max;
	*s=1-*s;
	
	double buf;

	switch(feeld){
	case 0:
		buf = max-min; if(buf<=0.0000001){buf=0.0000001;}
		*h=(60*((g-b)/buf));
		break;
	case 1:
		buf = max-min; if(buf<=0.0000001){buf=0.0000001;}
		*h=(60*((b-r)/buf))+120;
		break;
	case 2:
		buf = max-min; if(buf<=0.0000001){buf=0.0000001;}
		*h=(60*((r-g)/buf))+240;
		break;
	}
}






//-------------------------------------------------------------------------------
//
//	InitGlobals
//	
//	Initalize any global values here.  Called only once when global
//	space is reserved for the first time.
//
//	Inputs:
//		Ptr globalPtr		Standard pointer to a global structure.
//
//	Outputs:
//		Initializes any global values with their defaults.
//
//-------------------------------------------------------------------------------

void InitGlobals (Ptr globalPtr)
{	
	// create "globals" as a our struct global pointer so that any
	// macros work:
	GPtr globals = (GPtr)globalPtr;
	
	// Initialize global variables:
	gCount = 0;
	
} // end InitGlobals

//-------------------------------------------------------------------------------

// end NearestBase.cpp
